﻿using NUnit.Framework;
using PlatformSDKSample;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Threading;
using System.Net;
using System.IO;

namespace PlatformSDKSample.Tests
{
    [TestFixture()]
    public class PlatformSDKTests
    {
        private IntPtr loginHandle = IntPtr.Zero;    //station login handle
        private PlatformSDK.pfMessCallBack messCB = null;
        private GCHandle gcThis;

        [SetUp]
        public void InitialTest()
        {
            loginHandle = IntPtr.Zero;
            gcThis = GCHandle.Alloc(this);
        }

        [TearDown]
        public void ReleaseTest()
        {
            if (loginHandle != IntPtr.Zero)
            {
                PlatformSDK.Logout(ref loginHandle);
            }
        }

        private void SetLoginInfo(ref PlatformSDK.TLoginInfo loginInfo)
        {
            loginInfo.IP = "10.16.104.46";
            loginInfo.userName = "admin";
            loginInfo.password = "1";
            loginInfo.port = 3454;
            loginInfo.pfMessCB = messCB;
            loginInfo.context = (IntPtr)gcThis;
            loginInfo.timeoutSeconds = 20;
            loginInfo.blockLogin = true;
            loginInfo.heartBeatLostRetryTimes = 3;
            loginInfo.heartbeatPeriodSeconds = 5;
            loginInfo.VAST2WebSocketPort = 7768;
            loginInfo.useSSL = false;
        }

        [Test()]
        public void LoginSuccess()
        {
            PlatformSDK.TLoginInfo loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);

            uint ret = PlatformSDK.Login(ref loginHandle, ref loginInfo);

            Assert.AreEqual(0, ret);
        }

        [Test()]
        public void Login_InvalidHandle()
        {
            PlatformSDK.TLoginInfo loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);

            IntPtr handle = new IntPtr(1);
            uint ret = PlatformSDK.Login(ref handle, ref loginInfo);

            Assert.AreEqual(PlatformSDK.GENERAL_E_VNDP_INVALID_ARG, ret);
        }

        [Test()]
        public void Login_EmptyUserName()
        {
            PlatformSDK.TLoginInfo loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);

            loginInfo.userName = "";
            uint ret = PlatformSDK.Login(ref loginHandle, ref loginInfo);

            Assert.AreEqual(PlatformSDK.SERVER_CONNECTION_E_LOGIN_WITH_EMPTY_USERNAME, ret);
        }

        [Test()]
        public void Login_EmptyPassword()
        {
            PlatformSDK.TLoginInfo loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);

            loginInfo.password = "";
            uint ret = PlatformSDK.Login(ref loginHandle, ref loginInfo);

            Assert.AreEqual(PlatformSDK.SERVER_CONNECTION_E_LOGIN_WITH_EMPTY_PASSWORD, ret);
        }

        [Test()]
        public void GetRecordingDateList()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);

            var deviceListRequest = new PlatformSDK.TDeviceListRequest();
            var deviceList = new PlatformSDK.TRecordingDeviceList();

            deviceListRequest.stationID = null;
            deviceListRequest.timeoutMillisecond = 5000;

            PlatformSDK.GetRecordingDeviceList(loginHandle, ref deviceListRequest, ref deviceList);

            var request = new PlatformSDK.TRecordingDateReqInfo();
            var dateList = new PlatformSDK.TRecordingDateList();

            request.locationID = "Loc1";
            request.timeoutMillisecond = 5000;
            request.utcStartTimeSeconds = 0;        //1970-01-01
            request.utcEndTimeSeconds = 2082758399; //2035-12-31

            uint ret = PlatformSDK.GetRecordingDateList(loginHandle, ref request, ref dateList);

            Assert.AreEqual(0, ret);
            Assert.AreEqual(4, dateList.dateNum);
        }

        [Test()]
        public void GetRecordingDateList_InvalidLocID()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);

            var deviceListRequest = new PlatformSDK.TDeviceListRequest();
            var deviceList = new PlatformSDK.TRecordingDeviceList();

            deviceListRequest.stationID = null;
            deviceListRequest.timeoutMillisecond = 5000;

            PlatformSDK.GetRecordingDeviceList(loginHandle, ref deviceListRequest, ref deviceList);

            var request = new PlatformSDK.TRecordingDateReqInfo();
            var dateList = new PlatformSDK.TRecordingDateList();

            request.timeoutMillisecond = 5000;
            request.utcStartTimeSeconds = 0;        //1970-01-01
            request.utcEndTimeSeconds = 2082758399; //2035-12-31

            uint ret = PlatformSDK.GetRecordingDateList(loginHandle, ref request, ref dateList);

            Assert.AreEqual(PlatformSDK.GENERAL_E_VNDP_INVALID_ARG, ret);
        }        
        
        [Test()]
        public void GetRecordingDateList_SendRequestFail()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);

            var deviceListRequest = new PlatformSDK.TDeviceListRequest();
            var deviceList = new PlatformSDK.TRecordingDeviceList();

            deviceListRequest.stationID = null;
            deviceListRequest.timeoutMillisecond = 5000;

            PlatformSDK.GetRecordingDeviceList(loginHandle, ref deviceListRequest, ref deviceList);

            var request = new PlatformSDK.TRecordingDateReqInfo();
            var dateList = new PlatformSDK.TRecordingDateList();

            request.locationID = "Loc1<";
            request.timeoutMillisecond = 5000;
            request.utcStartTimeSeconds = 0;        //1970-01-01
            request.utcEndTimeSeconds = 2082758399; //2035-12-31

            uint ret = PlatformSDK.GetRecordingDateList(loginHandle, ref request, ref dateList);

            Assert.AreEqual(PlatformSDK.SERVER_CONNECTION_E_VNDP_TUNNEL_SEND_MESSAGE_FAIL, ret);
        }

        [Test()]
        public void GetDeviceListWithDetails_success()
        {
            LoginSuccess();

            PlatformSDK.TDeviceListRequest request = new PlatformSDK.TDeviceListRequest();
            PlatformSDK.TDeviceDetailListResponse response = new PlatformSDK.TDeviceDetailListResponse();
            int ret = PlatformSDK.GetDeviceListWithDetails(loginHandle, ref request, ref response);

            Assert.AreEqual(0, ret);

            Assert.AreEqual(17, response.deviceNum);
            PlatformSDK.TDeviceDetailInfo stationDeviceInfo = (PlatformSDK.TDeviceDetailInfo)Marshal.PtrToStructure(response.pDeviceList, typeof(PlatformSDK.TDeviceDetailInfo));
            Assert.AreEqual("VAST", stationDeviceInfo.modelName);
            Assert.IsTrue(stationDeviceInfo.firmwareVersion.StartsWith("VAST-VVTK-"));

            PlatformSDK.TDeviceDetailInfo firstCameraInfo = (PlatformSDK.TDeviceDetailInfo)Marshal.PtrToStructure(response.pDeviceList + Marshal.SizeOf(typeof(PlatformSDK.TDeviceDetailInfo)), typeof(PlatformSDK.TDeviceDetailInfo));
            Assert.AreEqual("FD8371", firstCameraInfo.modelName);
            Assert.IsTrue(firstCameraInfo.firmwareVersion.StartsWith("FD8371-VVTK-"));
            Assert.AreEqual("0002D18371C3", firstCameraInfo.mac);
            Assert.AreEqual(16384, firstCameraInfo.ptzCapability);
            Assert.AreEqual(0, firstCameraInfo.fisheyeCapability);
            Assert.AreEqual(4, firstCameraInfo.streamNum);
        }

        [Test()]
        public void GetDeviceListWithDetails_fail()
        {
            PlatformSDK.TDeviceListRequest request = new PlatformSDK.TDeviceListRequest();
            PlatformSDK.TDeviceDetailListResponse response = new PlatformSDK.TDeviceDetailListResponse();
            int ret = PlatformSDK.GetDeviceListWithDetails(loginHandle, ref request, ref response);

            Assert.AreNotSame(0, ret);
        }

        [Test()]
        public void GetPresetList_success()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);

            IntPtr pCamID = Marshal.StringToHGlobalAnsi("C_13");
            PlatformSDK.TSDKPresetList presetList = new PlatformSDK.TSDKPresetList();

            var ret = PlatformSDK.GetPresetList(loginHandle, pCamID, ref presetList);

            Assert.AreEqual(0, ret);
            Assert.AreEqual(2, presetList.dwPresetNum);
        }

        [Test()]
        public void GetPresetList_SendRequestFail()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);

            IntPtr pCamID = Marshal.StringToHGlobalAnsi("C_13<<");
            PlatformSDK.TSDKPresetList presetList = new PlatformSDK.TSDKPresetList();

            var ret = PlatformSDK.GetPresetList(loginHandle, pCamID, ref presetList);

            Assert.AreEqual(PlatformSDK.SERVER_CONNECTION_E_VNDP_TUNNEL_SEND_MESSAGE_FAIL, ret);
        }

        [Test()]
        public void GetPresetList_InvalidHandle()
        {
            IntPtr pCamID = Marshal.StringToHGlobalAnsi("C_13");
            PlatformSDK.TSDKPresetList presetList = new PlatformSDK.TSDKPresetList();

            var ret = PlatformSDK.GetPresetList(loginHandle, pCamID, ref presetList);

            Assert.AreEqual(PlatformSDK.GENERAL_E_VNDP_INVALID_HANDLE, ret);
        }

        [Test()]
        public void GetPresetList_InvalidArgument()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);

            IntPtr pCamID = IntPtr.Zero;
            PlatformSDK.TSDKPresetList presetList = new PlatformSDK.TSDKPresetList();

            var ret = PlatformSDK.GetPresetList(loginHandle, pCamID, ref presetList);

            Assert.AreEqual(PlatformSDK.GENERAL_E_VNDP_INVALID_ARG, ret);
        }

        [Test()]
        public void MoveToPreset_success()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);

            IntPtr pCamID = Marshal.StringToHGlobalAnsi("C_13");
            IntPtr pPresetName = Marshal.StringToHGlobalAnsi("P1");

            var ret = PlatformSDK.MoveToPreset(loginHandle, pCamID, pPresetName);

            Assert.AreEqual(0, ret);
        }

        [Test()]
        public void MoveToPreset_SendRequestFail()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);

            IntPtr pCamID = Marshal.StringToHGlobalAnsi("C_13<<<");
            IntPtr pPresetName = Marshal.StringToHGlobalAnsi("P1");

            var ret = PlatformSDK.MoveToPreset(loginHandle, pCamID, pPresetName);

            Assert.AreEqual(PlatformSDK.SERVER_CONNECTION_E_VNDP_TUNNEL_SEND_MESSAGE_FAIL, ret);
        }

        [Test()]
        public void MoveToPreset_InvalidHandle()
        {
            IntPtr pCamID = Marshal.StringToHGlobalAnsi("C_13");
            IntPtr pPresetName = Marshal.StringToHGlobalAnsi("P1");

            var ret = PlatformSDK.MoveToPreset(loginHandle, pCamID, pPresetName);

            Assert.AreEqual(PlatformSDK.GENERAL_E_VNDP_INVALID_HANDLE, ret);
        }

        [Test()]
        public void MoveToPreset_InvalidArgument()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);

            IntPtr pCamID = IntPtr.Zero;
            IntPtr pPresetName = Marshal.StringToHGlobalAnsi("P1");

            var ret = PlatformSDK.MoveToPreset(loginHandle, pCamID, pPresetName);

            Assert.AreEqual(PlatformSDK.GENERAL_E_VNDP_INVALID_ARG, ret);
        }

        [Test()]
        public void GetRecordingTimeList_success()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);
            
            PlatformSDK.TRecordingTimeReqInfo request = new PlatformSDK.TRecordingTimeReqInfo();
            PlatformSDK.TRecordingTimeList timeList = new PlatformSDK.TRecordingTimeList();

            string locationID = "loc1";
            request.utcStartTimeSeconds = 1551715200;
            request.utcEndTimeSeconds = 1551758120;
            request.locationID = locationID;
            request.timeoutMillisecond = 5000;

            PlatformSDK.GetRecordingTimeList(loginHandle, ref request, ref timeList);

            Assert.AreEqual(1, timeList.timeNum);

            var pTime = new IntPtr(timeList.timeList.ToInt64());
            var time = (PlatformSDK.TRecordingTime)Marshal.PtrToStructure(pTime, typeof(PlatformSDK.TRecordingTime));

            long startLocalSeconds = time.tServerTime.startSecond
                + time.tServerTime.startTimeZoneSecond
                + time.tServerTime.startDayLightSavingSecond;

            long endLocalSeconds = time.tServerTime.endSecond
                + time.tServerTime.endTimeZoneSecond
                + time.tServerTime.endDayLightSavingSecond;

            Assert.AreEqual(1551786802, startLocalSeconds);
            Assert.AreEqual(1551786919, endLocalSeconds);
        }

        [Test()]
        public void GetRecordingTimeList_invalidHandle()
        {
            PlatformSDK.TRecordingTimeReqInfo request = new PlatformSDK.TRecordingTimeReqInfo();
            PlatformSDK.TRecordingTimeList timeList = new PlatformSDK.TRecordingTimeList();

            string locationID = "loc1";
            request.utcStartTimeSeconds = 1551715200;
            request.utcEndTimeSeconds = 1551758120;
            request.locationID = locationID;
            request.timeoutMillisecond = 5000;

            var ret = PlatformSDK.GetRecordingTimeList(loginHandle, ref request, ref timeList);

            Assert.AreEqual(PlatformSDK.GENERAL_E_VNDP_INVALID_HANDLE, ret);
        }

        [Test()]
        public void GetRecordingTimeList_invalidArgument()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);

            PlatformSDK.TRecordingTimeReqInfo request = new PlatformSDK.TRecordingTimeReqInfo();
            PlatformSDK.TRecordingTimeList timeList = new PlatformSDK.TRecordingTimeList();

            string locationID = "loc1";
            request.utcStartTimeSeconds = 1551715200;
            request.utcEndTimeSeconds = 1551758120;
            request.locationID = "";
            request.timeoutMillisecond = 5000;

            var ret = PlatformSDK.GetRecordingTimeList(loginHandle, ref request, ref timeList);

            Assert.AreEqual(PlatformSDK.GENERAL_E_VNDP_INVALID_ARG, ret);
        }

        [Test()]
        public void GetRecordingTimeList_sendRequestFail()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);

            PlatformSDK.TRecordingTimeReqInfo request = new PlatformSDK.TRecordingTimeReqInfo();
            PlatformSDK.TRecordingTimeList timeList = new PlatformSDK.TRecordingTimeList();

            request.utcStartTimeSeconds = 1551715200;
            request.utcEndTimeSeconds = 1551758120;
            request.locationID = "loc1<<";
            request.timeoutMillisecond = 5000;

            var ret = PlatformSDK.GetRecordingTimeList(loginHandle, ref request, ref timeList);

            Assert.AreEqual(PlatformSDK.SERVER_CONNECTION_E_VNDP_TUNNEL_SEND_MESSAGE_FAIL, ret);
        }

        [Test()]
        public void MonitorEvent_success()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);
            var eventCB = new PlatformSDK.pfEventCallBack((IntPtr target) => { });

            var ret = PlatformSDK.MonitorEvent(loginHandle, eventCB, IntPtr.Zero);

            Assert.AreEqual(0, ret);
        }

        [Test()]
        public void MonitorEvent_invalidHandle()
        {
            var ret = PlatformSDK.MonitorEvent(loginHandle, null, IntPtr.Zero);

            Assert.AreEqual((int)PlatformSDK.GENERAL_E_VNDP_INVALID_HANDLE, ret);
        }

        [Test()]
        public void MonitorEvent_invalidArgument()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);

            var ret = PlatformSDK.MonitorEvent(loginHandle, null, IntPtr.Zero);

            Assert.AreEqual((int)PlatformSDK.GENERAL_E_VNDP_INVALID_ARG, ret);
        }

        [Test()]
        public void MonitorDeviceStatus_success()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);
            var eventCB = new PlatformSDK.pfDeviceStatusCallBack((IntPtr staticList, int staticNumber, IntPtr context) => {});

            var ret = PlatformSDK.MonitorDeviceStatus(loginHandle, eventCB, IntPtr.Zero);

            Assert.AreEqual(0, ret);
        }

        [Test()]
        public void MonitorDeviceStatus_invalidHandle()
        {
            var eventCB = new PlatformSDK.pfDeviceStatusCallBack((IntPtr staticList, int staticNumber, IntPtr context) => { });

            var ret = PlatformSDK.MonitorDeviceStatus(loginHandle, eventCB, IntPtr.Zero);

            Assert.AreEqual((int)PlatformSDK.GENERAL_E_VNDP_INVALID_HANDLE, ret);
        }

        [Test()]
        public void MonitorDeviceStatus_invalidArgument()
        {
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);

            var ret = PlatformSDK.MonitorDeviceStatus(loginHandle, null, IntPtr.Zero);

            Assert.AreEqual((int)PlatformSDK.GENERAL_E_VNDP_INVALID_ARG, ret);
        }
    }

    [TestFixture()]
    public class PlatformSDKTests_TunnelNotConnected
    {
        private IntPtr loginHandle = IntPtr.Zero;    //station login handle
        private PlatformSDK.pfMessCallBack messCB = null;
        private GCHandle gcThis;

        [SetUp]
        public void InitialTest()
        {
            loginHandle = IntPtr.Zero;
            gcThis = GCHandle.Alloc(this);
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);
        }

        [TearDown]
        public void ReleaseTest()
        {
            if (loginHandle != IntPtr.Zero)
            {
                PlatformSDK.Logout(ref loginHandle);
            }
        }

        private void SetLoginInfo(ref PlatformSDK.TLoginInfo loginInfo)
        {
            loginInfo.IP = "10.16.104.46";
            loginInfo.userName = "admin";
            loginInfo.password = "1";
            loginInfo.port = 3454;
            loginInfo.pfMessCB = messCB;
            loginInfo.context = (IntPtr)gcThis;
            loginInfo.timeoutSeconds = 20;
            loginInfo.blockLogin = false;
            loginInfo.heartBeatLostRetryTimes = 3;
            loginInfo.heartbeatPeriodSeconds = 5;
            loginInfo.VAST2WebSocketPort = 7768;
            loginInfo.useSSL = false;
        }

        [Test()]
        public void GetRecordingDateList_TunnelNotConnected()
        {
            var request = new PlatformSDK.TRecordingDateReqInfo();
            var dateList = new PlatformSDK.TRecordingDateList();

            request.locationID = "Loc1<<";
            request.timeoutMillisecond = 5000;
            request.utcStartTimeSeconds = 0;        //1970-01-01
            request.utcEndTimeSeconds = 2082758399; //2035-12-31

            uint ret = PlatformSDK.GetRecordingDateList(loginHandle, ref request, ref dateList);

            Assert.AreEqual(PlatformSDK.SERVER_CONNECTION_E_QUERY_SERVER_NOT_LOGIN, ret);
        }

        [Test()]
        public unsafe void GetLiveAbnormalEventList_TunnelNotConnected()
        {
            int abnormalEventNum = 0;
            IntPtr pAbnormalEventList = IntPtr.Zero;

            uint ret = PlatformSDK.GetLiveAbnormalEventList(loginHandle, ref pAbnormalEventList, &abnormalEventNum);

            Assert.AreEqual(PlatformSDK.SERVER_CONNECTION_E_CONFIG_SERVER_NOT_LOGIN, ret);
        }

        [Test()]
        public unsafe void DeleteAbnormalEvent_TunnelNotConnected()
        {
            string deviceID = "/Event/system/0/volume_error/0";
            IntPtr pDeviceID = Marshal.StringToHGlobalAnsi(deviceID);

            uint ret = PlatformSDK.DeleteAbnormalEvent(loginHandle, pDeviceID);
            Assert.AreEqual(PlatformSDK.SERVER_CONNECTION_E_CONFIG_SERVER_NOT_LOGIN, ret);
        }

        [Test()]
        public unsafe void DeleteAllAbnormalEvent_TunnelNotConnected()
        {
            uint ret = PlatformSDK.DeleteAllAbnormalEvent(loginHandle);
            Assert.AreEqual(PlatformSDK.SERVER_CONNECTION_E_CONFIG_SERVER_NOT_LOGIN, ret);
        }

        [Test()]
        public void GetRecordingTimeList_TunnelNotConnected()
        {
            PlatformSDK.TRecordingTimeReqInfo request = new PlatformSDK.TRecordingTimeReqInfo();
            PlatformSDK.TRecordingTimeList timeList = new PlatformSDK.TRecordingTimeList();

            request.utcStartTimeSeconds = 1551715200;
            request.utcEndTimeSeconds = 1551758120;
            request.locationID = "loc1";
            request.timeoutMillisecond = 5000;

            var ret = PlatformSDK.GetRecordingTimeList(loginHandle, ref request, ref timeList);

            Assert.AreEqual(PlatformSDK.SERVER_CONNECTION_E_QUERY_SERVER_NOT_LOGIN, ret);
        }

        [Test()]
        public void MonitorEvent_TunnelNotConnected()
        {
            var ret = PlatformSDK.MonitorEvent(loginHandle, null, IntPtr.Zero);

            Assert.AreEqual((int)PlatformSDK.SERVER_CONNECTION_E_EVENT_SERVER_NOT_LOGIN, ret);
        }

        [Test()]
        public void MonitorDeviceStatus_TunnelNotConnected()
        {
            var eventCB = new PlatformSDK.pfDeviceStatusCallBack((IntPtr staticList, int staticNumber, IntPtr context) => { });

            var ret = PlatformSDK.MonitorDeviceStatus(loginHandle, eventCB, IntPtr.Zero);

            Assert.AreEqual((int)PlatformSDK.SERVER_CONNECTION_E_EVENT_SERVER_NOT_LOGIN, ret);
        }
    }

    [TestFixture()]
    public class PlatformSDKTests_SECOM_NVR
    {
        private IntPtr loginHandle = IntPtr.Zero;    //station login handle
        private PlatformSDK.pfMessCallBack messCB = null;
        private GCHandle gcThis;
        private List<PlatformSDK.TEventInfo> eventList = new List<PlatformSDK.TEventInfo>();

        [SetUp]
        public void InitialTest()
        {
            loginHandle = IntPtr.Zero;
            gcThis = GCHandle.Alloc(this);
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);
        }

        [TearDown]
        public void ReleaseTest()
        {
            if (loginHandle != IntPtr.Zero)
            {
                PlatformSDK.Logout(ref loginHandle);
            }
        }

        private void SetLoginInfo(ref PlatformSDK.TLoginInfo loginInfo)
        {
            loginInfo.IP = "10.16.105.85";
            loginInfo.userName = "admin";
            loginInfo.password = "aaaa1111";
            loginInfo.port = 3454;
            loginInfo.pfMessCB = messCB;
            loginInfo.context = (IntPtr)gcThis;
            loginInfo.timeoutSeconds = 20;
            loginInfo.blockLogin = true;
            loginInfo.heartBeatLostRetryTimes = 3;
            loginInfo.heartbeatPeriodSeconds = 5;
            loginInfo.VAST2WebSocketPort = 7768;
            loginInfo.useSSL = false;
        }

        [Test()]
        public void GetDeviceListWithDetails_success()
        {

            PlatformSDK.TDeviceListRequest request = new PlatformSDK.TDeviceListRequest();
            PlatformSDK.TDeviceDetailListResponse response = new PlatformSDK.TDeviceDetailListResponse();
            int ret = PlatformSDK.GetDeviceListWithDetails(loginHandle, ref request, ref response);

            Assert.AreEqual(0, ret);

            PlatformSDK.TDeviceDetailInfo stationDeviceInfo = (PlatformSDK.TDeviceDetailInfo)Marshal.PtrToStructure(response.pDeviceList, typeof(PlatformSDK.TDeviceDetailInfo));
            Assert.AreEqual("NV-R0030", stationDeviceInfo.modelName);
            Assert.IsTrue(stationDeviceInfo.firmwareVersion.StartsWith("ND9541P-SECO-"));
            Assert.AreEqual(8, stationDeviceInfo.DINum);
            Assert.AreEqual(4, stationDeviceInfo.DONum);
        }

        [Test()]
        public unsafe void GetLiveAbnormalEventList_success()
        {
            int abnormalEventNum = 0;
            IntPtr pAbnormalEventList = IntPtr.Zero;

            uint ret = PlatformSDK.GetLiveAbnormalEventList(loginHandle, ref pAbnormalEventList, &abnormalEventNum);

            Assert.AreEqual(0, ret);
        }

        [Test()]
        public unsafe void DeleteAllAbnormalEvent_success()
        {
            uint ret = PlatformSDK.DeleteAllAbnormalEvent(loginHandle);
            Assert.AreEqual(0, ret);
        }

        [Test()]
        public unsafe void DeleteAbnormalEvent_success()
        {
            string deviceID = "/Event/system/0/volume_error/0";
            IntPtr pDeviceID = Marshal.StringToHGlobalAnsi(deviceID);

            uint ret = PlatformSDK.DeleteAbnormalEvent(loginHandle, pDeviceID);
            Assert.AreEqual(0, ret);
        }

        [Test()]
        public unsafe void DeleteAbnormalEvent_invalidId()
        {
            string deviceID = "0";
            IntPtr pDeviceID = Marshal.StringToHGlobalAnsi(deviceID);

            uint ret = PlatformSDK.DeleteAbnormalEvent(loginHandle, pDeviceID);
            Assert.AreEqual(PlatformSDK.GENERAL_E_FAIL, ret);
        }
        
        void EventCallBack(IntPtr pEventListInfo)
        {
            PlatformSDK.TEventListInfo info = (PlatformSDK.TEventListInfo)Marshal.PtrToStructure(pEventListInfo, typeof(PlatformSDK.TEventListInfo));

            for (uint i = 0; i < info.eventNum; ++i)
            {
                IntPtr pEventInfo = new IntPtr(info.pEventList.ToInt64() + Marshal.SizeOf(typeof(PlatformSDK.TEventInfo)) * i);
                PlatformSDK.TEventInfo eventInfo = (PlatformSDK.TEventInfo)Marshal.PtrToStructure(pEventInfo, typeof(PlatformSDK.TEventInfo));
                eventList.Add(eventInfo);
            }
        }

        [Test()]
        public void SetNVRDO_Low()
        {
            int ret = PlatformSDK.SetNVRDO(loginHandle, 1, 1);
            var eventCB = new PlatformSDK.pfEventCallBack(EventCallBack);

            ret = PlatformSDK.MonitorEvent(loginHandle, eventCB, IntPtr.Zero);
            eventList.Clear();
            ret = PlatformSDK.SetNVRDO(loginHandle, 1, 0);
            Thread.Sleep(1000);

            Assert.AreEqual(true, eventList.Exists(eventInfo => eventInfo.eventType == PlatformSDK.EEventType.DO &&
                    eventInfo.deviceType == PlatformSDK.EDeviceType.station &&
                    eventInfo.index == 1 &&
                    eventInfo.trigger == 0));
        }

        [Test()]
        public void SetNVRDO_High()
        {
            int ret = PlatformSDK.SetNVRDO(loginHandle, 1, 0);
            var eventCB = new PlatformSDK.pfEventCallBack(EventCallBack);

            ret = PlatformSDK.MonitorEvent(loginHandle, eventCB, IntPtr.Zero);
            eventList.Clear();
            ret = PlatformSDK.SetNVRDO(loginHandle, 1, 1);
            Thread.Sleep(1000);

            Assert.AreEqual(true, eventList.Exists(eventInfo => eventInfo.eventType == PlatformSDK.EEventType.DO &&
                    eventInfo.deviceType == PlatformSDK.EDeviceType.station &&
                    eventInfo.index == 1 &&
                    eventInfo.trigger == 1));
        }

        [Test()]
        public unsafe void QueryAlarm()
        {
            var request = new PlatformSDK.TQueryAlarmRequest();

            request.maxRecords = 5000;
            request.startTime = 1575504000;
            request.endTime = 1575504000 + 86400;

            int alarmRecordNum = 0;
            IntPtr pAlarmRecords = IntPtr.Zero;

            int ret = PlatformSDK.QueryAlarm(loginHandle, ref request, ref pAlarmRecords, &alarmRecordNum);

            Thread.Sleep(1000);

            Assert.AreEqual(2078, alarmRecordNum);
        }

        [Test()]
        public unsafe void QueryAlarm_WithLimit()
        {
            var request = new PlatformSDK.TQueryAlarmRequest();

            request.maxRecords = 100;
            request.startTime = 1575504000;
            request.endTime = 1575504000 + 86400;

            int alarmRecordNum = 0;
            IntPtr pAlarmRecords = IntPtr.Zero;

            int ret = PlatformSDK.QueryAlarm(loginHandle, ref request, ref pAlarmRecords, &alarmRecordNum);

            Thread.Sleep(1000);

            Assert.AreEqual(100, alarmRecordNum);
        }

        [Test()]
        public void GetRecordingTimeList_success()
        {
            PlatformSDK.TRecordingTimeReqInfo request = new PlatformSDK.TRecordingTimeReqInfo();
            PlatformSDK.TRecordingTimeList timeList = new PlatformSDK.TRecordingTimeList();

            request.utcStartTimeSeconds = 1575849600;
            request.utcEndTimeSeconds = 1575862560;
            request.locationID = "loc1";
            request.timeoutMillisecond = 5000;

            PlatformSDK.GetRecordingTimeList(loginHandle, ref request, ref timeList);

            Assert.AreEqual(1, timeList.timeNum);

            var pTime = new IntPtr(timeList.timeList.ToInt64());
            var time = (PlatformSDK.TRecordingTime)Marshal.PtrToStructure(pTime, typeof(PlatformSDK.TRecordingTime));

            long startLocalSeconds = time.tServerTime.startSecond
                + time.tServerTime.startTimeZoneSecond
                + time.tServerTime.startDayLightSavingSecond;

            long endLocalSeconds = time.tServerTime.endSecond
                + time.tServerTime.endTimeZoneSecond
                + time.tServerTime.endDayLightSavingSecond;

            Assert.AreEqual(1575849600, startLocalSeconds);
            Assert.AreEqual(1575862560, endLocalSeconds);
        }

        [Test()]
        public unsafe void QueryNVRSystemLog()
        {
            var request = new PlatformSDK.TQueryNVRLogRequest();

            request.maxRecords = 5000;
            request.startTime = 1575849600;
            request.endTime = 1575849600 + 86400;
            request.logType = PlatformSDK.ENVRLogType.system;

            int logNVRRecordNum = 0;
            IntPtr pLogNVRRecords = IntPtr.Zero;

            int ret = PlatformSDK.QueryNVRLogList(loginHandle, ref request, ref pLogNVRRecords, &logNVRRecordNum);

            Thread.Sleep(1000);

            Assert.AreEqual(46, logNVRRecordNum);
        }

        [Test()]
        public unsafe void QueryNVRRecordingLog()
        {
            var request = new PlatformSDK.TQueryNVRLogRequest();

            request.maxRecords = 5000;
            request.startTime = 1575849600;
            request.endTime = 1575849600 + 86400;
            request.logType = PlatformSDK.ENVRLogType.recording;

            int logNVRRecordNum = 0;
            IntPtr pLogNVRRecords = IntPtr.Zero;

            int ret = PlatformSDK.QueryNVRLogList(loginHandle, ref request, ref pLogNVRRecords, &logNVRRecordNum);

            Thread.Sleep(1000);

            Assert.AreEqual(166, logNVRRecordNum);
        }

        [Test()]
        public unsafe void QueryNVRAbnormalEventLog()
        {
            var request = new PlatformSDK.TQueryNVRLogRequest();

            request.maxRecords = 5000;
            request.startTime = 1575849600;
            request.endTime = 1575849600 + 86400;
            request.logType = PlatformSDK.ENVRLogType.abnormalEvent;

            int logNVRRecordNum = 0;
            IntPtr pLogNVRRecords = IntPtr.Zero;

            int ret = PlatformSDK.QueryNVRLogList(loginHandle, ref request, ref pLogNVRRecords, &logNVRRecordNum);

            Thread.Sleep(1000);

            Assert.AreEqual(32, logNVRRecordNum);
        }

        [Test()]
        public unsafe void QueryNVRErrorLog()
        {
            var request = new PlatformSDK.TQueryNVRLogRequest();

            request.maxRecords = 5000;
            request.startTime = 1575849600;
            request.endTime = 1575849600 + 86400;
            request.logType = PlatformSDK.ENVRLogType.error;

            int logNVRRecordNum = 0;
            IntPtr pLogNVRRecords = IntPtr.Zero;

            int ret = PlatformSDK.QueryNVRLogList(loginHandle, ref request, ref pLogNVRRecords, &logNVRRecordNum);

            Thread.Sleep(1000);

            Assert.AreEqual(3, logNVRRecordNum);
        }

        [Test()]
        public unsafe void QueryNVRUserLog()
        {
            var request = new PlatformSDK.TQueryNVRLogRequest();

            request.maxRecords = 5000;
            request.startTime = 1575849600;
            request.endTime = 1575849600 + 86400;
            request.logType = PlatformSDK.ENVRLogType.user;

            int logNVRRecordNum = 0;
            IntPtr pLogNVRRecords = IntPtr.Zero;

            int ret = PlatformSDK.QueryNVRLogList(loginHandle, ref request, ref pLogNVRRecords, &logNVRRecordNum);

            Thread.Sleep(1000);

            Assert.AreEqual(17, logNVRRecordNum);
        }

        [Test()]
        public unsafe void QueryNVRLog_WithLimit()
        {
            var request = new PlatformSDK.TQueryNVRLogRequest();

            request.maxRecords = 5;
            request.startTime = 1575849600;
            request.endTime = 1575849600 + 86400;
            request.logType = PlatformSDK.ENVRLogType.user;

            int logNVRRecordNum = 0;
            IntPtr pLogNVRRecords = IntPtr.Zero;

            int ret = PlatformSDK.QueryNVRLogList(loginHandle, ref request, ref pLogNVRRecords, &logNVRRecordNum);

            Thread.Sleep(1000);

            Assert.AreEqual(5, logNVRRecordNum);
        }
    }

    //
    // Prepare camera that id is C_1 and ip is 172.18.150.48.
    [TestFixture]
    public class PlatformSDKTests_NVR_PTZ
    {
        private IntPtr loginHandle = IntPtr.Zero;    //station login handle
        private PlatformSDK.pfMessCallBack messCB = null;
        private GCHandle gcThis;
        private PlatformSDK.TContinuousPTZRequest continuousPTZRequest;

        [SetUp]
        public void InitialTest()
        {
            loginHandle = IntPtr.Zero;
            gcThis = GCHandle.Alloc(this);
            var loginInfo = new PlatformSDK.TLoginInfo();
            SetLoginInfo(ref loginInfo);
            PlatformSDK.Login(ref loginHandle, ref loginInfo);
            SetupContinuousPTZCamera();
            PlatformSDK.StopPTZControl(loginHandle, ref continuousPTZRequest);
        }

        [TearDown]
        public void ReleaseTest()
        {
            if (loginHandle != IntPtr.Zero)
            {
                PlatformSDK.StopPTZControl(loginHandle, ref continuousPTZRequest);
                PlatformSDK.Logout(ref loginHandle);
            }
        }

        private void SetLoginInfo(ref PlatformSDK.TLoginInfo loginInfo)
        {
            loginInfo.IP = "10.16.105.85";
            loginInfo.userName = "admin";
            loginInfo.password = "aaaa1111";
            loginInfo.port = 3454;
            loginInfo.pfMessCB = messCB;
            loginInfo.context = (IntPtr)gcThis;
            loginInfo.timeoutSeconds = 20;
            loginInfo.blockLogin = true;
            loginInfo.heartBeatLostRetryTimes = 3;
            loginInfo.heartbeatPeriodSeconds = 5;
            loginInfo.VAST2WebSocketPort = 7768;
            loginInfo.useSSL = false;
        }

        private void SetupContinuousPTZCamera()
        {
            continuousPTZRequest.cameraID = "C_1";
            continuousPTZRequest.streamIndex = 1;
            continuousPTZRequest.channelIndex = 1;
        }

        private void SetPTZDirectionAndSpeed()
        {
            continuousPTZRequest.speed = 1;
            continuousPTZRequest.x = 1;
            continuousPTZRequest.y = 0;
        }

        private string getPanFromWeb()
        {
            WebClient client = new WebClient();
            client.Credentials = new NetworkCredential("root", "tapas");
            Stream stream = client.OpenRead("http://172.18.150.48/cgi-bin/camctrl/camctrl.cgi?getpan");
            StreamReader reader = new StreamReader(stream);
            return reader.ReadToEnd();
        }

        [Test()]
        public void ContinuousPTZControl()
        {
            string beforePosition = getPanFromWeb();

            SetPTZDirectionAndSpeed();
            PlatformSDK.ContinuousPTZControl(loginHandle, ref continuousPTZRequest);
            Thread.Sleep(1000);

            string afterPosition = getPanFromWeb();

            Assert.AreNotEqual(beforePosition, afterPosition);
        }

        [Test()]
        public void StopPTZControl()
        {
            SetPTZDirectionAndSpeed();
            PlatformSDK.ContinuousPTZControl(loginHandle, ref continuousPTZRequest);
            Thread.Sleep(1000);

            PlatformSDK.StopPTZControl(loginHandle, ref continuousPTZRequest);
            Thread.Sleep(500);
            string beforePosition = getPanFromWeb();

            Thread.Sleep(1000);
            string afterPosition = getPanFromWeb();

            Assert.AreEqual(beforePosition, afterPosition);
        }
    }
}